﻿'import statements help reduce line length and typing
Imports MccDaq
Imports MccDaq.ChannelType
Imports MccDaq.Range
Imports MccDaq.ScanOptions
Imports MccDaq.DigitalPortType
Imports MccDaq.FunctionType
Imports MccDaq.ErrorInfo.ErrorCode
Imports MccDaq.MccService
Imports System.Convert
Imports System.Threading.Thread
Imports System.Timers
Imports System.IO

Module Module1

    Const BLOCKSIZE As Integer = 100 'samples per channel
    Const CHANNELCOUNT As Integer = 4
    Const SAMPLERATE As Integer = 100 'desired sample rate
    Const BUFFERSIZE As Integer = CHANNELCOUNT * BLOCKSIZE ' size of the buffer
    Const HALFBUFFER As Integer = BUFFERSIZE / 2 'size of half buffer
    Const TOTAL As Integer = SAMPLERATE * CHANNELCOUNT * 60 * 2 '2 minutes

    'AInScan running options
    Dim theOptions As MccDaq.ScanOptions = Continuous + Background + ConvertData


    Dim CurrentCount As Integer = 0 'indicates total samples taken
    Dim CurrentIndex As Integer = 0 'points to the last scan read

    Dim Status As Short = 0

    Dim theDevice As MccBoard 'device object
    Dim buffer As IntPtr
    Dim ret As ErrorInfo
    Dim ReadLower As Boolean = True 'semaphore flag that prevents duplicate buffer reads

    'text file name and path. Use ASC extension if using DASYLab software to read the file.
    Dim Path As String = "C:\Users\Public\Documents\theDataFile.asc"

    Dim fStream As StreamWriter

    Dim ChanArray(CHANNELCOUNT) As Short                    ' array to hold channel queue information
    Dim ChanTypeArray(CHANNELCOUNT) As MccDaq.ChannelType   ' array to hold channel type information
    Dim GainArray(CHANNELCOUNT) As MccDaq.Range             ' array to hold gain queue information
    Dim ChanText(CHANNELCOUNT) As String
    Dim ChanScale(CHANNELCOUNT) As Single



    Sub Main()

        Dim atimer As Timer
        Dim ret As MccDaq.ErrorInfo
        Dim bNum As Integer
        Dim str As String = "USB-1616HS"

        bNum = GetBoardNum(str)

        If bNum = -1 Then
            Console.WriteLine("No {0} detected!", str)
            WaitForKey()
            End
        End If

 

        'load the arrays with values
        ChanArray(0) = 0
        ChanTypeArray(0) = Analog
        GainArray(0) = Bip10Volts
        ChanText(0) = "In0 [V]" + vbTab
        ChanScale(0) = Convert.ToSingle(10.0 / 32768) 'volts per bit for the +/- 10 volt range

        ChanArray(1) = 1
        ChanTypeArray(1) = Analog
        GainArray(1) = Bip10Volts
        ChanText(1) = "In1 [V]" + vbTab
        ChanScale(1) = Convert.ToSingle(10.0 / 32768) 'volts per bit for the +/- 10 volt range

        ChanArray(2) = 3
        ChanTypeArray(2) = Analog
        GainArray(2) = Bip10Volts
        ChanText(2) = "In1 [V]" + vbTab
        ChanScale(2) = Convert.ToSingle(10.0 / 32768) 'volts per bit for the +/- 10 volt range

        ChanArray(3) = FirstPortA
        ChanTypeArray(3) = Digital16
        GainArray(3) = NotUsed
        ChanText(3) = "FirstPortA [dec]"
        ChanScale(3) = 1


        theDevice = New MccBoard(bNum) 'get the devices' programming object

        'set digital port C for output
        ret = theDevice.DConfigPort(FirstPortC, MccDaq.DigitalPortDirection.DigitalOut)
        If ret.Value <> MccDaq.ErrorInfo.ErrorCode.NoErrors Then
            Stop
        End If

        'let's start with all 8 bits low
        Dim PortVal As UShort = 0
        ret = theDevice.DOut(FirstPortC, PortVal)
        If ret.Value <> MccDaq.ErrorInfo.ErrorCode.NoErrors Then
            Stop
        End If

        buffer = MccService.WinBufAllocEx(BUFFERSIZE)
        atimer = New Timer
        atimer.AutoReset = True
        atimer.Interval = 50
        AddHandler atimer.Elapsed, AddressOf TimerEvent

        Console.WriteLine("This program reads channel 1,2,3 & FirstPortA as a 16 bit word." + vbCrLf)
        Console.WriteLine("The program will stream the data continously to:")
        Console.WriteLine("{0}{1}", Path, vbCrLf)
        Console.WriteLine("While doing this the digital output lines on Port C can be changed as follows:" + vbCrLf)

        Console.WriteLine("Keys 0 - 3 toggles the state of D0-D3 on FirstPortC." + vbCrLf)
        Console.WriteLine("Press <SPACE> to clear all...<ESC> exit program" + vbCrLf)
        WaitForKey()


        ret = theDevice.DaqInScan(ChanArray, ChanTypeArray, GainArray, CHANNELCOUNT, SAMPLERATE, 0, BUFFERSIZE, buffer, theOptions)
        If ret.Value <> MccDaq.ErrorInfo.ErrorCode.NoErrors Then Stop

        'create a text file to hold the data
        fStream = New StreamWriter(Path)
        CreateFileHeader()

        'Timer routines checks/reads data every 50mS
        atimer.Start()

        '''''''''''''''''''' MAIN LOOP ''''''''''''''''''''''''''
        Dim cki As ConsoleKeyInfo = New ConsoleKeyInfo
        Do

            If Console.KeyAvailable = True Then
                cki = Console.ReadKey()
                'call function to update digital outputs
                UpdateFirstPortC(cki)
            End If

        Loop Until ((cki.Key = ConsoleKey.Escape) Or (CurrentCount > TOTAL))
        ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
        atimer.Stop()
        WaitForKey()
        theDevice.StopBackground(AiFunction)
        fStream.Close()
    End Sub

    Sub TimerEvent(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs)


        ret = theDevice.GetStatus(Status, CurrentCount, CurrentIndex, MccDaq.FunctionType.DaqiFunction)
        If ret.Value <> MccDaq.ErrorInfo.ErrorCode.NoErrors Then Stop

        'ping pong between low half and upper half
        If ((CurrentIndex >= HALFBUFFER) And (ReadLower = True)) Then

            'read the lower half and set Upper flag to zero so that next read is upper half
            Dim theArray(BUFFERSIZE) As UShort
            ret = MccDaq.MccService.WinBufToArray(buffer, theArray, 0, HALFBUFFER)
            If ret.Value <> MccDaq.ErrorInfo.ErrorCode.NoErrors Then Stop

            ReadLower = False
            DisplayData(theArray, HALFBUFFER \ CHANNELCOUNT)

        ElseIf ((CurrentIndex < HALFBUFFER) And (ReadLower = False)) Then

            'read the upper half and set Upper flag to one so that next read is lower half
            Dim theArray(BUFFERSIZE) As UShort
            ret = MccDaq.MccService.WinBufToArray(buffer, theArray, HALFBUFFER, HALFBUFFER)
            If ret.Value <> MccDaq.ErrorInfo.ErrorCode.NoErrors Then Stop

            ReadLower = True
            DisplayData(theArray, HALFBUFFER \ CHANNELCOUNT)

        End If

    End Sub

    Sub UpdateFirstPortC(ByVal cki As ConsoleKeyInfo)
        'Function used to change level on individual FirstPortC lines

        Dim PortVal As UShort
        Dim ret As ErrorInfo = New ErrorInfo

        Select Case cki.Key

            Case ConsoleKey.D0

                ret = theDevice.DIn(FirstPortC, PortVal)
                PortVal = PortVal Xor 1
                ret = theDevice.DOut(DigitalPortType.FirstPortC, PortVal)

            Case ConsoleKey.D1

                ret = theDevice.DIn(FirstPortC, PortVal)
                PortVal = PortVal Xor 2
                ret = theDevice.DOut(DigitalPortType.FirstPortC, PortVal)

            Case ConsoleKey.D2

                ret = theDevice.DIn(FirstPortC, PortVal)
                PortVal = PortVal Xor 4
                ret = theDevice.DOut(DigitalPortType.FirstPortC, PortVal)

            Case ConsoleKey.D3

                ret = theDevice.DIn(FirstPortC, PortVal)
                PortVal = PortVal Xor 8
                ret = theDevice.DOut(DigitalPortType.FirstPortC, PortVal)

            Case ConsoleKey.Spacebar
                ret = theDevice.DOut(DigitalPortType.FirstPortC, 0)

        End Select
        If ret.Value <> MccDaq.ErrorInfo.ErrorCode.NoErrors Then Stop

    End Sub


    'This function is dedicated to printing the program channels to the console screen
    'Helper function is dedicated to printing and writing to file the data
    Sub DisplayData(ByRef theArray() As UShort, ByVal rows As Integer)

        Static Scan As Integer = 0
        Dim index As Integer = 0

        Dim i As Integer = 0
        Dim j As Integer = 0
        Dim temp(CHANNELCOUNT) As Single
        Try

            For i = 0 To rows - 1

                Dim str As String = ""

                Scan += 1 'sample counter

                For j = 0 To CHANNELCOUNT - 1
                    If ChanTypeArray(j) = Digital16 Then
                        temp(j) = theArray(index)
                        str = str + temp(j).ToString("00000") + vbTab
                        index += 1
                    Else
                        temp(j) = ((theArray(index) - 32767) * ChanScale(j))
                        str = str + temp(j).ToString("0.00") + vbTab
                        index += 1
                    End If
                Next j
                str.Trim(vbTab)
                Console.WriteLine(Scan.ToString + vbTab + str)
                fStream.WriteLine(str)
            Next i
            fStream.Flush()


        Catch e As Exception
            MsgBox(e.Message)
        End Try

    End Sub


    'helper function that waits for any key to be pressed
    Sub WaitForKey()
        Dim cki As ConsoleKeyInfo
        Console.WriteLine("Press any key to continue..." + vbCrLf)
        Do
            System.Threading.Thread.Sleep(200)
        Loop Until (Console.KeyAvailable)
        cki = Console.ReadKey()
        Console.WriteLine("")

    End Sub


    'this is a helper function that writes header information to the screen and text file
    Sub CreateFileHeader()


        ''''''''''''''''''''' create text file header strings ''''''''''''''''
        Dim hdr(5) As String
        hdr(0) = "Recording date     : {0},  {1}"
        hdr(1) = "Block length       : {0}"
        hdr(2) = "Delta              : {0} sec."
        hdr(3) = "Number of channels : {0}"

        ''''''''''''''''''''''' build column headers string '''''''''''''''''''
        'example hdr(4) = "In0 [V]" + vbTab + "In1 [V]" + vbTab + "In1 [V]"
        For j = 0 To CHANNELCOUNT - 1
            hdr(4) = hdr(4) + ChanText(j)
        Next j
        hdr(4).Trim(vbTab)
        '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
        Dim MyDate As Date = Date.Now

        fStream.WriteLine(hdr(0), MyDate.ToString("d"), TimeString)
        fStream.WriteLine(hdr(1), HALFBUFFER \ CHANNELCOUNT)
        fStream.WriteLine(hdr(2), (1 / SAMPLERATE))
        fStream.WriteLine(hdr(3), CHANNELCOUNT)
        fStream.WriteLine(hdr(4))

        Console.WriteLine(hdr(0), MyDate.ToString("d"), TimeString)
        Console.WriteLine(hdr(1), HALFBUFFER \ CHANNELCOUNT)
        Console.WriteLine(hdr(2), (1 / SAMPLERATE))
        Console.WriteLine(hdr(3), CHANNELCOUNT)
        Console.WriteLine(hdr(4))
    End Sub


    'Helper function uses a string identifier to locate the devices board number. If more than one device is present then
    'it returns the first one matching the identifier.
    Function GetBoardNum(ByVal devString As String) As Integer

        Dim BoardNum As Integer
        For BoardNum = 0 To 99
            Dim DaqBoard As MccDaq.MccBoard
            DaqBoard = New MccDaq.MccBoard(BoardNum)
            If DaqBoard.BoardName.Contains(devString) Then
                Console.WriteLine("{0} board number = {1}" + vbCrLf, devString, BoardNum.ToString())
                DaqBoard.FlashLED()
                Return BoardNum
            End If

        Next BoardNum
        Return -1
    End Function

End Module
